;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xbd290efeffbf03037ad0adc09d521af2 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x4522d52fe69b8ae2b208d4cf23f68593 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xaa8f3647351cfde9248dfbbb65502b5c . "./code.cg2js/cg-grid-widget.fasl")
(#xe3cf2a8af9b5d8417c9729f3b8cdde5e . "./code.cg2js/cg-utility-dialog.fasl")
(#x09bbeac2f3fef91e40f0bddc52133ee7 . "./code.cg2js/cg-wrap-string.fasl")
(#x97deb0044a1bf751080fe4a7cedd9328 . "./code.cg2js/cg-editable-text.fasl")
(#x8d43960c1e2d5c67b9e35e30e75171a3 . "./code.cg2js/cg-base.fasl")
(#x15869621f945b55aaa611d57d9aa2567 . "./code.cg2js/cg-lisp-text.fasl")
(#x3f78859c3ad9dc5a68b794899811818f . "./code.cg2js/cg-intersect.fasl")
(#x228030a25e071eda813190a146b0f55c . "./code.cg2js/cg-choose-printer.fasl")
(#x4a5575bf7f3d947f6b10d39420f216b4 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x02bad7dfee6f60f652a5b0fbafa0732e . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x95045d8b45c3d292226630cf2d9176c5 . "./code.cg2js/cg-font-dialog.fasl")
(#xf6a97b6c84a3ad6d7f54042852391b51 . "./code.cg2js/cg-comtab.fasl")
(#x13bc37e6effec351f7b0dddc6a90b151 . "./code.cg2js/cg-tooltip.fasl")
(#x72d727961a0d7205fc1b98568cfb7943 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x161e1ec1feedc2eef7770ac90d18aa95 . "./code.cg2js/cg-get-pixmap.fasl")
(#x13cc768d27c71daa011a60d64c3c7f83 . "./code.cg2js/cg-status-bar.fasl")
(#x542ef15107c8fa28b2b4b70cef703e31 . "./code.cg2js/cg-tab-control.fasl")
(#xdffe2a0ecc283c614e3e7e31a8d7564f . "./code.cg2js/cg-icon-pixmap.fasl")
(#x43f8ff03e9014db6f32b57e8d3602fbf . "./code.cg2js/cg-list-view.fasl")
(#xd8d44581aae3553f8fe09d6294d83f73 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x81bb6edb47b9c4edbb03e25f0fea3003 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x88837ea2296e0b4925bcd09166fecbe8 . "./code.cg2js/cg-project-window.fasl")
(#x9a650373cd530af3e5b2840139bd238c . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x046a9a56c1c8c128981fc68996805637 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xdab5f3313aefce8f5087a28c6b543283 . "./code.cg2js/cg-scaling-stream.fasl")
(#x1a8fa7aea7c2a62ef206f13a061dc78e . "./code.cg2js/cg-bitmap-stream.fasl")
(#xd517a76fe0190b4777910fa9b3f488f0 . "./code.cg2js/cg-menu.fasl")
(#x7c67e7448f492833c3e96f1661578950 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x0b7880fae7ae810da3c08b57fef96847 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd492309d6f7dee273e278a4366d002d4 . "./code.cg2js/cg-split-bar.fasl")
(#xe2ac7e722a6245b3e6bbe68734f46753 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xbfcc91d616cec693dfcc1d43040b1947 . "./code.cg2js/cg-timer.fasl")
(#x5c7730f5caec150fe7ed2b3df7ff1cd1 . "./code.cg2js/cg-text-or-combo.fasl")
(#x131771fe24637f8e713a10ab40e3f9ce . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x9b0e8e9300d54b3ee2e30a5fff8aa412 . "./code.cg2js/cg-string-dialog.fasl")
(#x723da565197d29cf055e00a826931b5c . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xf9828afe36adf2294804bc377976be83 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x7846605278bb458b3249b55498a4b2c1 . "./code.cg2js/cg-outline.fasl")
(#xdb69e8760af28c821a93251a6bffc71b . "./code.cg2js/cg-lettered-menu.fasl")
(#x6ab80cf8064dce7a3c6358e45b9a5e6a . "./code.cg2js/cg-graphics-context.fasl")
(#x1fda60f3f49ffa0f4e43e666f8092649 . "./code.cg2js/cg-item-list.fasl")
(#x557e2ba78d8c1fc9c4f2dc1259119a53 . "./code.cg2js/cg-caret.fasl")
(#xef651803ed86bef8c2236cbe31da31c8 . "./code.cg2js/cg-printing.fasl")
(#x16d414dfa2d99a8ca413b1c03722ceef . "./code.cg2js/cg-pixmap-printing.fasl")
(#xde27d9a12492327eabc238c88859d8a9 . "./code.cg2js/cg-progress-indicator.fasl")
(#x736fa7eb1829dba5865e4ea3efe16c76 . "./code.cg2js/cg-dde-utils.fasl")
(#xab2138624394ff14b8095a7b5e92a59c . "./code.cg2js/cg-plot-widget.fasl")
(#x76203741cb692e581f575dcfb4fa065d . "./code.cg2js/cg-os-window.fasl")
(#x8c01722c3df3bf58912b6aac26fa0abd . "./code.cg2js/cg-calendar.fasl")
(#xc9befc21503dbf151ff7386948b21fe6 . "./code.cg2js/cg-chart-widget.fasl")
(#x742b430adf33353770953c7efc6a4fee . "./code.cg2js/cg-file-dialog.fasl")
(#xc13fa1f78b3d1858772ba05866940b0c . "./code.cg2js/cg-nodes.fasl")
(#xe56caaad29ba3b60703fef0299fb100d . "./code.cg2js/cg-lamp.fasl")
(#x5f8f7c5870ec0508f31061dcb5017649 . "./code.cg2js/cg-html-dialog.fasl")
(#x6f557f2e70eead949c05ca1f085720ea . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x65b076179b20c860d2faff98ca721609 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x2da4bc4258961d546a3c232b83657c37 . "./code.cg2js/cg-choice-list.fasl")
(#x7c6c94199539cbaf8c48d01f84765c38 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x88651ad63fad77be47324c8019e22fc6 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xa73a2d0bce80385562db16ad9a0fce44 . "./code.cg2js/cg-hotspot.fasl")
(#x36c9757f8c1ffdf7c0a2fedd3893927d . "./code.cg2js/cg-property.fasl")
(#xb5c44eb1da6caf9e547e5bffd36b465a . "./code.cg2js/cg-dropping-outline.fasl")
(#x694c57c39100566d17f94ebfa8cb467c . "./code.cg2js/cg-template-string.fasl")
(#xb8825e25e82d4946fa1dac57c33066c7 . "./code.cg2js/cg-palette.fasl")
(#x241c665df491cae64edfb421e4994be9 . "./code.cg2js/cg-selected-object.fasl")
(#xefc1c9a53f251601ff93d940d9ed8912 . "./code.cg2js/cg-up-down-control.fasl")
(#xb70daaad682d998c0050439fc98cad09 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x881df42db98b9c8085ed6de41a703702 . "./code.cg2js/cg-layout.fasl")
(#x4e8ff1042e5ae6ddf60e7248cc3a21e6 . "./code.cg2js/cg-get-position.fasl")
(#xa2b64c58f0022b3aae2cd38656c51599 . "./code.cg2js/cg-lisp-widget.fasl")
(#x5193e1f37c595f7c646f27249d134028 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x58eec0430f0af675e961e771a2ffdfe4 . "./code.cg2js/cg-object-editor.fasl")
(#xbe29f4af20f488dba2eda07b4cabff05 . "./code.cg2js/cg-class-grid.fasl")
(#xd2f3482f870c237b429f78ce2772b0c2 . "./code.cg2js/cg-yes-no-string.fasl")
(#x2a4810498d8cf357cacb9ebe855f5648 . "./code.cg2js/cg-toggling-widget.fasl")
(#xc65a806011714354775665e4889236c3 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x13a7a136df8c7f83e719aa7e89740c49 . "./code.cg2js/cg-button.fasl")
(#xea4483dc8c49fad2fd32ffe4dc31e057 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x4311c28f20709d7b398687498c3182f9 . "./code.cg2js/cg-scroll-bar.fasl")
(#x1cef81d4eb07a0c5123afd5b54c51830 . "./code.cg2js/cg-combo-box.fasl")
(#x4aac2f3c863e7b676f484f4396099994 . "./code.cg2js/cg-web-browser.fasl")
(#x5ca3df2b6f54e6adebcf20f32b388632 . "./code.cg2js/cg-drawable.fasl")
(#x1ca72ab7704105ab02d8d20dfd035dde . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x3e17f0b765942145a0d382a00bd17cab . "./code.cg2js/cg-message-dialog.fasl")
(#xacfd4d4161eb79d7783ce77450b65b2d . "./code.cg2js/cg-text-widget.fasl")
(#x1bd55974dc4878ad6c92b3520a69f84b . "./code.cg2js/cg-static-text.fasl")
(#x9ea5f426f2989839358a96f0b7814f69 . "./code.cg2js/cg-class-support.fasl")
(#xbc207df283e84b2b9842bacd8f019d50 . "./code.cg2js/cg.fasl")
(#x524ebc1c2ebedd8f4c3a70765481f073 . "./code.cg2js/cg-directory-dialog.fasl")
(#xf790fda7af71bb23aa2558176ea36faf . "./code.cg2js/cg-clipboard.fasl")
(#x3c1798e3efea952ff3d48ceddd3d9422 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xa590d4febff4fca7781b6a740932e96d . "./code.cg2js/cg-bitmap-pane.fasl")
(#x124e441871507528358c05b12b70781d . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x6a95606e5eaf54610aebd2d1a75f4795 . "./code.cg2js/cg-check-box.fasl")
(#xb73df84776eb68c0f0b6ca008815ce5f . "./code.cg2js/cg-color-dialog.fasl")
(#xd159e248f51f980eab70da9b9656592d . "./code.cg2js/cg-text-edit-pane.fasl")
(#xf8b73bec1071458b92d9dcc2d344a3f7 . "./code.cg2js/cg-dialog-item.fasl")
(#xd350c2b04c021e509d857d94719c1d2b . "./code.cg2js/cg-edit-in-place.fasl")
(#xc3664f969bd8fa35b37cb575bec4de69 . "./code.cg2js/cg-tray.fasl")
(#x1a61627a9d7919600722438a9c3f6e12 . "./code.cg2js/cg-paren-matching.fasl")
(#x45c40a033727444c5287b0f0e06ac72b . "./code.cg2js/cg-player.fasl")
(#x772493b677874e359991b9923b04409d . "./code.cg2js/cg-yes-no-list.fasl")
(#x1863d08ad31d97d1a69209c218b187ed . "./code.cg2js/cg-common-control.fasl")
(#x885331d41a7b9757dbe4f62aff8d8dca . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x89ce98ec49f1bc41eb4d216b224eab08 . "./code.cg2js/cg-os-widget.fasl")
(#x5da290795c86a2b3ffd1531ca84ee97f . "./code.cg2js/cg-sample-file-menu.fasl")
(#x5ffe2278138870dd00f5b8864ae9aa40 . "./code.cg2js/cg-trackbar.fasl")
(#xce48c03e1946423eb852086294b9c1d7 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xd1fbead3784127dca2b9a434dbeb5747 . "./code.cg2js/cg-html-widget.fasl")
(#xbdddb934b26ea7830f12e165c29a6376 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xbec6ed33a17aeb3a1c4b4fc4d9b82fd6 . "./code.cg2js/cg-acache.fasl")
(#xd96bbd1e626250e1aadc693758120db9 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x5240e792f50e4307eaa6b8174ea454a6 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x0f9ccfc35b06996f190dd963541f681e . "./code.cg2js/cg-icon.fasl")
(#xfdad8d713cabbe6aa4344f0275a28cfe . "./code.cg2js/cg-pixmap.fasl")
(#x41b2a79b352f165cddb782a90e4fe031 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x99187af23795f87dc3759025a4d46f73 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x9daebe6d3f2fb7113f1d85b43f43436b . "./code.cg2js/cg-toolbar.fasl")
(#xe1a2cef02288de4e580966914df1fb9c . "./code.cg2js/cg-radio-button.fasl")
(#xa3eb81dd327641185865eed898294439 . "./code.cg2js/cg-curve.fasl")
(#x2c920f243d967ae6cb19ec20164eacfa . "./code.cg2js/cg-picture-widget.fasl")
(#x2aabfa08a9f32f6a08e9795ac8c9f4e4 . "./code.cg2js/cg-dev.fasl")
(#xde956dd4635ddb01b53db0ddd364137e . "./code.cg2js/cg-group-box.fasl")
(#xed6c310b7ad90448d33b6e93e91a62fa . "./code.cg2js/cg-fill-texture.fasl")
(#x8644fdeaf3993e010774cb3a2b47b0ff . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x4d2c01cff45a0ee023eddf1e6c0d52e2 . "./code.idejs/ide-keyshort.fasl")
(#x81353f44569375e14cc0700e2a70493c . "./code.idejs/ide-form.fasl")
(#x5240f495e2627f26310e29b63c70deac . "./code.idejs/ide-find-in-files.fasl")
(#x72c9583ee901192886ac240efb36901d . "./code.idejs/ide-class-grapher.fasl")
(#x36a647cdcf3193b693aa062cf73ef16c . "./code.idejs/ide-base.fasl")
(#x341f7d0397ea1d75db19c5498329a3ae . "./code.idejs/ide-class-browser.fasl")
(#x9f2e489234794e12c6314339600922b8 . "./code.idejs/ide-find-definitions.fasl")
(#x756a262d16319c0f231358e850bd4f5a . "./code.idejs/ide-package-list.fasl")
(#x5db36367c1701aba7910246026e7cdd3 . "./code.idejs/ide-grapher.fasl")
(#xa153ba10678a18b272bfbffd343bd3c7 . "./code.idejs/ide-dev.fasl")
(#xf8dbba43a1ac9db50aa601d6edf9927d . "./code.idejs/ide.fasl")
(#x9923283cbc50ad35acb6289659721e8c . "./code.idejs/ide-trace.fasl")
(#xeeadb345e1125afc6c3db7656540732b . "./code.idejs/ide-apropos.fasl")
(#x411f88ad9b407fd1147d96171c30a046 . "./code.idejs/ide-inspector.fasl")
(#xc89ea76a4152c05a633eb8788890544c . "./code.idejs/ide-acache.fasl")
(#x2c7ce6b33832867dc8c1928a3ae4470e . "./code.idejs/ide-project.fasl")
(#x1a44284ca201250c7d1e997f4fda0f46 . "./code.idejs/ide-window-list.fasl")
(#x193d44e9600bdfaa22e98fbe191057c1 . "./code.idejs/ide-intro.fasl")
(#xef87889aebf79a6a690cb5b09fcc53c4 . "./code.idejs/ide-clipboard.fasl")
(#x604278f0cf2c51c134e8ddca954f0a4b . "./code.idejs/ide-editor.fasl")
(#x74a676ad1ec9da6695e3b9e74ee34ada . "./code.idejs/ide-process-browser.fasl")
(#x0b4f9a8c2259db3d27b034535dd25ea6 . "./code.idejs/ide-patch-dialog.fasl")
(#x964719c154325a6f2e0e7f6233b9e9b7 . "./code.idejs/ide-navigator.fasl")
(#x7e8a653f63c59e56c07642baa40b4a7f . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
